package models

import (
	"time"

	"github.com/jackc/pgtype"
	"orus.io/orus-io/go-orusapi/database"
)

// dbtable:"email_queue" dbpkey:"id"
type Email struct {
	ID            int              `db:"id"`
	CreatedAt     time.Time        `db:"created_at"`
	From          string           `db:"from_"`
	To            pgtype.TextArray `db:"to_"`
	Msg           []byte           `db:"msg"`
	StatusChanged time.Time        `db:"status_changed"`
	Status        string           `db:"status"`
	LastError     string           `db:"last_error"`
	Attempts      int              `db:"attempts"`
	ReattemptAt   time.Time        `db:"reattempt_at"`
}

func QueueMail(db database.SQLHelper, from string, to []string, msg []byte) error {
	email := Email{
		CreatedAt:     time.Now().UTC(),
		From:          from,
		Msg:           msg,
		Status:        "pending",
		StatusChanged: time.Now().UTC(),
	}
	if err := email.To.Set(to); err != nil {
		return err
	}

	if _, err := db.InsertNoPKey(&email); err != nil {
		return err
	}

	if _, err := db.ExecRaw("NOTIFY email_queue_notify"); err != nil {
		return err
	}

	return nil
}
