package models

import (
	"io"
	"slices"
)

func (m *Metadata) Add(name, value string) {
	if *m == nil {
		*m = make(Metadata)
	}

	(*m)[name] = value
}

func (m Metadata) Remove(name ...string) Metadata {
	for _, n := range name {
		delete(m, n)
	}

	return m
}

func (m Metadata) Clone() Metadata {
	out := make(Metadata)
	for key, value := range m {
		out[key] = value
	}

	return out
}

func (m Metadata) Update(entries Metadata) Metadata {
	if m == nil {
		panic("cannot update 'nil' Metadata")
	}

	for key, value := range entries {
		m[key] = value
	}

	return m
}

func (m Metadata) Keys() []string {
	var i int

	keys := make([]string, len(m))
	for k := range m {
		keys[i] = k
		i++
	}

	slices.Sort(keys)

	return keys
}

func (m Metadata) writeTo(w io.Writer) error {
	for _, k := range m.Keys() {
		v := m[k]

		if _, err := w.Write([]byte(k)); err != nil {
			return err
		}

		if _, err := w.Write([]byte("=")); err != nil {
			return err
		}

		if _, err := w.Write([]byte(v)); err != nil {
			return err
		}

		if _, err := w.Write([]byte("\n")); err != nil {
			return err
		}
	}

	return nil
}
